/**
 * This navbar.js features the behavior of the nav bar, the menu linked to it, and the
 *  back-to-top button.
 */

'use strict';

(function() {
  window.addEventListener('load', init);

  let resist = false; // whether the menu toggling is stopped
  let updateResistTimeoutId = null;

  function init() {
    window.addEventListener('scroll', checkTopBtn);
    id('nav-btn').addEventListener('click', toggleMenu);
    let primaryItems = qsa('#primary ol li');
    for (let i = 0; i < primaryItems.length; i++) {
      primaryItems[i].addEventListener('mouseenter', displayHover);
    }
    id('menu').addEventListener('click', attemptToggleMenu);
    let disks = qsa('.disk');
    for (let i = 0; i < disks.length; i++) {
      disks[i].addEventListener('click', resistToggleMenu);
    }
    id("topBtn").addEventListener('click', backToTop);

  }

  /**
   * attempts to close the menu. The attemp only succeeds if the operation is not stopped.
   */
  function attemptToggleMenu() {
    setTimeout(() => {
      if (!resist) {
        toggleMenu();
      }
    }, 50);
  }

  /**
   * Stops the operation of toggling the menu. The resistance will be reset after 0.1s.
   */
  function resistToggleMenu() {
    resist = true;
    clearTimeout(updateResistTimeoutId);
    updateResistTimeoutId = setTimeout(() => {
      resist = false;
    }, 100);
  }

  /**
   * Toggles the menu of the navbar to either appear or disappear.
   */
  function toggleMenu() {
    id('nav-btn').classList.toggle('triggered');
    id('menu').classList.toggle('triggered');
    let menuDisks = qsa('.disk');
    for (let i = 0; i < menuDisks.length; i++) {
      menuDisks[i].classList.toggle('triggered');
    }
    clearHover();
  }

  /**
   * displays a visual effect whenever an particular element is hovered.
   */
  function displayHover() {
    clearHover();
    this.classList.add('hovered');
    // check which one it is at
    let index = 0;
    let curr = this;
    while (curr !== null) {
      index += 1;
      curr = curr.previousElementSibling;
    }
    let correspond = qs('#secondary nav:nth-child('+index+')');
    if (correspond !== null) { // in case the element does not exist
      correspond.classList.remove('hidden');
    }
  }

  /**
   * clears all the hover visual effects on the screen.
   */
  function clearHover() {
    let primaryItems = qsa('#primary ol li');
    for (let i = 0; i < primaryItems.length; i++) {
      primaryItems[i].classList.remove('hovered');
    }
    let secondaries = qsa('#secondary nav');
    for (let i = 0; i < secondaries.length; i++) {
      secondaries[i].classList.add('hidden');
    }
  }

  /**
   * checks whether to display the top button. The button will show up
   *  when the user has scrolled more than 20 pixels.
   */
  function checkTopBtn() {
    let topBtn = id("topBtn");
    if (document.body.scrollTop > 20 || document.documentElement.scrollTop > 20) {
      topBtn.classList.remove("concealed");
    } else {
      topBtn.classList.add("concealed");
    }
  }

  /**
   * scroll to the top of the document when the user clicks on the button
   */
  function backToTop() {
    document.body.scrollTop = 0;
    document.documentElement.scrollTop = 0;
  }

  /**
   * Returns the element that has the ID attribute with the specified value.
   * @param {string} idName - element ID
   * @returns {object} DOM object associated with id.
   */
  function id(idName) {
    return document.getElementById(idName);
  }

  /**
   * Returns the first element that matches the given CSS selector.
   * @param {string} selector - CSS query selector.
   * @returns {object} The first DOM object matching the query.
   */
  function qs(selector) {
    return document.querySelector(selector);
  }

  /**
   * Returns the array of elements that match the given CSS selector.
   * @param {string} selector - CSS query selector
   * @returns {object[]} array of DOM objects matching the query.
   */
  function qsa(selector) {
    return document.querySelectorAll(selector);
  }
})();